(defvar ctl-t-map (make-keymap)
  "Keymap for subcommands of C-t")
(fset 'ctl-t-prefix ctl-t-map)

(if (eq window-system 'x) (load "~/.emacs.X") (load "~/.emacs.noX"))

(require 'desktop)   ; desktop loading & saving
(require 'cc-mode)   ; hm..

(setq sentence-end-double-space nil); don't require double space to end sentence..
;(setq inhibit-eol-conversion t)     ; don't assume dos compatibility mode
(setq tags-revert-without-query t)  ; TAGS files are reverted automatically
(standard-display-european t)       ;  should work after this
(show-paren-mode t)                 ; How to show matching parenthesis.
(menu-bar-mode -1)                  ; no menubar, thanks
(column-number-mode t)              ; show column number in status bar
(setq vc-follow-symlinks t)         ; cvs & rcs : follow symlinks
(set 'inhibit-startup-message t)    ; no trash in the initial scratch buffer
(resize-minibuffer-mode 1)          ; resize minibuffer when it doesn't fit one line
(setq transient-mark-mode nil)      ; don't show marked region
(setq next-line-add-newlines nil)   ; when moving down with the cursor, don't pass end of buffer
(setq compilation-ask-about-save nil) ; always save all files before compiling
(setq ediff-diff-options "-wd")      ; ignore whitespace in diffs
(setq diff-program "diff -wd")
(setq user-full-name "Elmer J. Fudd")
(setq user-mail-address "elmer@fudd.com")
(setq compilation-window-height 6)  ; compilation window is 6 rows.
(setq desktop-globals-to-save       ; some additional variables to save..
      (cons 'compilation-search-path
	    (cons 'compile-command
		  desktop-globals-to-save)))

;(autoload 'html-helper-mode "html-helper-mode" "Yay HTML" t)
(autoload 'sawfish-mode "sawfish" "sawfish-mode" t)
(autoload 'ruby-mode "ruby-mode" "ruby-mode" t)

(setq auto-mode-alist (cons '("\\.jl$" . sawfish-mode) auto-mode-alist))
(setq auto-mode-alist (cons '("\\.wml$" . sgml-mode) auto-mode-alist))
(setq auto-mode-alist (cons '("\\.sawfishrc$" . sawfish-mode) auto-mode-alist))
(setq auto-mode-alist (cons '("\\.ebuild$" . sh-mode) auto-mode-alist))

(setq interpreter-mode-alist (cons '("ruby" . ruby-mode) interpreter-mode-alist))

(c-add-style "java-2" '("java" (c-basic-offset . 2) (c-offsets-alist (case-label . +)) (c-hanging-comment-ender-p . nil))) ; java-tab width 2, not 4

(add-hook 'c-mode-hook '(lambda () (setq c-file-style "ellemtel")))   ; I like the ellemtel style..
(add-hook 'c++-mode-hook '(lambda () (setq c-file-style "ellemtel")))
(add-hook 'java-mode-hook '(lambda () (setq c-file-style "java-2"))) ; use my java-2 style defined above

;; the following stuff is for the pp-buffer* functions, which "prints" files to postscript files

(setq ps-underlined-faces '())
(setq ps-landscape-mode t)
(setq ps-number-of-columns 3)
(setq ps-font-size 5.1)
(setq ps-top-margin 29)
(setq ps-bottom-margin 29)
(setq ps-left-margin 29)
(setq ps-right-margin 29)
(setq ps-inter-column 29)
(setq ps-italic-faces '(italic bold-italic firebrick-italic red-bold-italic))
(setq ps-bold-faces '(bold bold-italic red-bold-italic ForestGreen-bold red-bold blue-bold))
(setq ps-paper-type 'a4)

(defun pp-buffer (fname) "Exports a buffer to a nice postscript file"
  (interactive "FDump PS to file: ")
  (setq ps-print-color-p nil)
  (ps-print-buffer fname)
  )

(defun pp-buffer-color (fname) "Exports a buffer to a nice color postscript file"
  (interactive "FDump PS to file: ")
  ;; should possibly insert commands here to adjust the colors used in output
  (setq ps-print-color-p t)
  (ps-print-buffer-with-faces fname)
  )


(defun save-all-buffers () "Saves all unsaved buffers"
  (interactive)
  (save-some-buffers t)
  )

(defun desktop-read-kill-scratch ()
  "Kills the *scratch* buffer and reads the desktop"
  (interactive)
  (let ((x (buffer-list)))
    (while x
      (if (equal (buffer-name (car x)) "*scratch*")
	  (kill-buffer "*scratch*"))
      (setq x (cdr x))))
  (desktop-read)
  )


;; jeah, buffer switching:

					;(defun buffer-vname (x)
					;  (let ((name (buffer-file-name x)))
					;    (if (null name) (buffer-name x) name)))

(defun buffer-lessp (x y) (string-lessp (buffer-name x) (buffer-name y)))
(defun buffer-equalp (x y) (string-equal (buffer-name x) (buffer-name y)))

(defun sort-bufferlist (lst)
  (let ((firstelem (car lst)) (sortedlst (sort lst 'buffer-lessp)))
    (rotate sortedlst firstelem 'buffer-equalp)))

(defun rotate (lst srch equalp)
  "rotates a list such that the given 'srch' element is first in the list"
  (defun fe (curlst found)
    (if (null curlst)
	(cons '() '())
      (let ((newfound (or (funcall equalp (car curlst) srch) found)))
	(let ((ret (fe (cdr curlst) newfound)))
	  (if newfound
	      (cons '() (cons (car curlst) (cdr ret)))
	    (cons (cons (car curlst) (car ret)) (cdr ret)))))))
  (let ((x (fe lst '())))
    (append (cdr x) (car x))))

(defun strip-buffers (x)
  "Strip off all 'unwanted' buffers from given list of buffers"
  (let ((result nil))
    (while x
      (let ((name (buffer-name (car x))))
	(if (and
	     (or
	      (string-lessp name " *Minibuf");; ignore minibuffers
	      (string-lessp " *Minibug" name))
	     (or
	      (string-lessp name " *code-convert");; ignore code converting buffers
	      (string-lessp " *code-converu" name))
	     (or
	      (string-lessp name "*");; ignore uppercase buffers starting with *
	      (string-lessp "*Z" name))
	     (or
	      (string-lessp name " *");; ignore uppercase buffers starting with  *
	      (string-lessp " *Z" name))
	     (not
	      (string-equal name "*vc*"));; ignore *vc* buffer
	     (not
	      (string-equal name "*desktop*"));; ignore *desktop* buffer
	     (or
	      (string-lessp name "TAGS");; ignore TAGS named buffers
	      (string-lessp "TAGT" name)))
	    (setq result (append result (cons (car x) nil))))
	(setq x (cdr x))))
    (sort-bufferlist result)))


(defun previous-buffer ()
  "Switch to the previous buffer (the last buffer in the buffer list)"
  (interactive)
  (let ((buf (cadr (strip-buffers (buffer-list)))))
    (if (not (null buf))
	(switch-to-buffer buf))))
;; these are not used now that I got alpha-sorted buffer-switching to work
					;  (let ((x (reverse (cdr (strip-buffers (buffer-list))))))
					;    (while x
					;      (switch-to-buffer (car x))
					;      (setq x (cdr x)))))

(defun next-buffer ()
  "Switch to the next buffer (the first buffer in the buffer list)"
  (interactive)
  (let ((buf (car (last (strip-buffers (buffer-list))))))
    (if (not (null buf))
	(switch-to-buffer buf))))

(defun open-java11-src ()
  "Open the directory containing the java1.1 source code into dired"
  (interactive)
  (dired "/opt/java/jdk1.1src/java"))

(defun open-java2-src ()
  "Open the directory containing the java2 source code into dired"
  (interactive)
  (dired "/opt/java/jdk1.2src/java"))

(defun hscroll-mode-wrapper ()
  "Wrapper for the old hscroll-mode, which does the appropriate thing according to version."
  (interactive)
  (if (string-lessp "21" emacs-version)
      (toggle-truncate-lines)
    (hscroll-mode)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;
;;;;;;;;;  KEYBOARD
;;;;;;;;

;;  some keys are also defined in .emacs.X !!!

					; disable non-break-space
(defun insert-space () (interactive) (insert " "))
(global-set-key [160] 'insert-space)

					; no prefix
(define-key global-map "\^t"  'ctl-t-prefix)
(define-key global-map "\^h"  'backward-delete-char-untabify)
(define-key global-map "\^\\" 'help-command)
(define-key global-map "\^xs" 'save-all-buffers)
;; (define-key global-map "\^xb" 'iswitchb-buffer)
(define-key global-map [?\C-,] 'next-buffer)
(define-key global-map [?\C-.] 'previous-buffer)
(define-key global-map [C-M-prior] 'previous-error)
(define-key global-map [C-M-next] 'next-error)
(define-key global-map [M-up]   '(lambda () (interactive) (scroll-other-window -1)))
(define-key global-map [M-down] '(lambda () (interactive) (scroll-other-window 1)))
(define-key global-map [S-return] 'newline)
(define-key global-map [S-backspace] 'backward-delete-char-untabify)
(define-key global-map [home] 'beginning-of-buffer)
(define-key global-map [C-home] 'beginning-of-buffer)
(define-key global-map [end] 'end-of-buffer)
(define-key global-map [C-end] 'end-of-buffer)

(define-key global-map [C-mouse-4] 'scroll-down)
(define-key global-map [C-mouse-5] 'scroll-up)

; M-  prefix (also for escape sequences)
(define-key esc-map   "\^r" 'rename-buffer)
(define-key esc-map   "A"   'append-to-file)
(define-key esc-map   "OD"  'backward-char)
(define-key esc-map   "OC"  'forward-char)
(define-key esc-map   "OB"  'next-line)
(define-key esc-map   "OA"  'previous-line)
(define-key esc-map   "-"   'dabbrev-expand)
					;(define-key esc-map   [left]  'next-buffer)
					;(define-key esc-map   [right]  'previous-buffer)
					;(define-key esc-map   [up]  'next-buffer)
					;(define-key esc-map   [down]  'previous-buffer)

					; C-t prefix
(define-key ctl-t-map "+"   'gamma-hi)	; brighter colors
(define-key ctl-t-map "-"   'gamma-lo)	; dimmer colors (default)
(define-key ctl-t-map "\^t" 'transpose-chars) ; the original C-t command
(define-key ctl-t-map "c"   'compile)	; run make or similar program
(define-key ctl-t-map "d"   'gdb)	; run make or similar program
(define-key ctl-t-map "f"   'small-font) ; switch to smaller font (800x600 suitable)
(define-key ctl-t-map "F"   'normal-font) ; switch to normal font (1024x768 suitable)
(define-key ctl-t-map "g"   'goto-line)	; jump to a line
(define-key ctl-t-map "i"   'indent-region) ; indent from mark to cursor location
(define-key ctl-t-map "j"   'open-java11-src) ; open java1.1 source
(define-key ctl-t-map "\^j" 'open-java2-src) ; open java2 source
(define-key ctl-t-map "k"   'delete-frame) ; delete the current frame (unless last frame)
(define-key ctl-t-map "l"   'desktop-read-kill-scratch) ; read all files mentioned in .emacs.desktop
(define-key ctl-t-map "L"   'desktop-save) ; save currently open files to .emacs.desktop
(define-key ctl-t-map "m"   'make-frame) ; make a new frame on current display
(define-key ctl-t-map "M"   'make-frame-on-display) ; make a new frame on some display
(define-key ctl-t-map "\^m" 'man)	; read manpages
;;;;;;;;;(define-key ctl-t-map "n"   'gnus)                      ; gnus mail & newsreader
(define-key ctl-t-map "r"   'revert-buffer)             ; reload file from disk, discarding changes
(define-key ctl-t-map "\^r" 'reverse-region)            ; reverse the order of lines
(define-key ctl-t-map "s"   'sort-lines)                ; sort the lines in current region
(define-key ctl-t-map "\^s" 'server-start)              ; start the server for emacsclient
(define-key ctl-t-map "v"   'reverse-region)            ; reverse the lines in current region
(define-key ctl-t-map "w"   'nuke-trailing-whitespace)  ; kill whitespace at end-of-line and end-of-buffer
(define-key ctl-t-map "\^w" 'hscroll-mode-wrapper)      ; wrap too long lines or not ?

;; the desktop stuff works such that when you have once saved the desktop with desktop-save
;; or loaded it with desktop-load, then after that the desktop is automatically saved when
;; you exit emacs

					;(define-key ctl-t-map "h" ctl-t-h-map)

					; enable some functions

(put 'eval-expression 'disabled nil)	; M-:
(put 'downcase-region 'disabled nil)	; C-x C-l
(put 'upcase-region 'disabled nil)	; C-x C-u

					; IDL colorization

(font-lock-add-keywords
 'idl-mode
 `(("^#[        ]*error[        ]+\\(.+\\)" 1 'font-lock-warning-face prepend)
   ("^#[        ]*\\(include\\)[        ]+\\(<[^>\"\n]*>?\\)" 2 'font-lock-string-face)
   ("^#[        ]*define[       ]+\\(\\sw+\\)(" 1 'font-lock-function-name-face)
   ("^#[        ]*\\(elif\\|if\\)\\>"
    ("\\<\\(defined\\)\\>[      ]*(?\\(\\sw+\\)?" nil nil
     (1 'font-lock-reference-face)
     (2 'font-lock-variable-name-face nil t)))
   ("\\(__declspec\\)[  ]*(\\([^)]+\\))"
    (1 'font-lock-reference-face)
    (2 'font-lock-variable-name-face))
   ("^#[        ]*\\(\\sw+\\)\\>[       ]*\\(\\sw+\\)?"
    (1 'font-lock-reference-face)
    (2 'font-lock-variable-name-face nil t))
   ("\\<\\(raises\\)\\>" 1 'font-lock-keyword-face)
   ("[  ]*\\([A-Za-z][A-Za-z0-9_]*\\)[  ]*(" 1 'font-lock-function-name-face)
   ("\\<\\(any\\|boolean\\|char\\|const\\|double\\|enum\\|fixed\\|float\\|interface\\|long\\|module\\|native\\|octet\\|Object\\|sequence\\|short\\|string\\|struct\\|unsigned\\|union\\|void\\|wchar\\|wstring\\)\\>" 1 'font-lock-type-face)
   ("\\<\\(attribute\\|case\\|context\\|default\\|exception\\|FALSE\\|in\\|inout\\|oneway\\|out\\|readonly\\|switch\\|TRUE\\|typedef\\)\\>" 1 'font-lock-keyword-face)) 'set)
(add-hook 'idl-mode-hook '(lambda () (font-lock-mode 1)))
